# Installer Functions
function OverwriteShortcut ($shortcutObject) {
  if($shortcutObject.execute -eq $true) {
    $tempShortcutPathAndName = "" + $shortcutObject.linkPath + "\" + $shortcutObject.linkName
    Write-Debug "This is the Shortcut Full Path:"; Write-Debug $tempShortcutPathAndName
    Install-ChocolateyShortcut -ShortcutFilePath $tempShortcutPathAndName -TargetPath $shortcutObject.destPath
  }
}

function OverwriteSymlink ($symlinkObject) {
  if($symlinkObject.execute -eq $true) {
    $tempSymlinkPathAndName = $symlinkObject.linkPath + "\" + $symlinkObject.linkName
    if (Test-Path -Path $symlinkObject.destPath) {
      Write-Debug ("Destination Path " + $symlinkObject.destPath + " already exists.")
    } else {
      Write-Debug ("Creating " + $symlinkObject.destPath)
      New-Item $symlinkObject.destPath -ItemType Directory -Force
    }
    if (Test-Path -Path $tempSymlinkPathAndName) {
      if((Get-Item $tempSymlinkPathAndName).Attributes.ToString() -Match "ReparsePoint") {
        Write-Debug ("Link Path " + $tempSymlinkPathAndName + " already exists as SymbolicLink. Will overwrite SymbolicLink.")
      } else {
        Write-Debug ("Link Path " + $tempSymlinkPathAndName + " already exists. Will move to destination path and link afterwards.")
        Move-Item -Path $tempSymlinkPathAndName -Destination $symlinkObject.destPath -Force
      }
    }
    Write-Debug ("Linking " + $tempSymlinkPathAndName + " to " + $symlinkObject.destPath)
    New-Item -Type SymbolicLink -Path "$tempSymlinkPathAndName" -Value $symlinkObject.destPath -Force
  }
}

function CreateRegKey ($regKeyObject) {
  if($regKeyObject.execute -eq $true) {
    if($regKeyObject.key -eq '') {
      New-Item -Path ${regKeyObject}.path -Value ${regKeyObject}.value -Force
    } else {
      if (Test-Path ${regKeyObject}.path) { } else { New-Item ${regKeyObject}.path -force }
      New-ItemProperty -Type String -Path ${regKeyObject}.path -Name ${regKeyObject}.key -Value ${regKeyObject}.value -force
    }
  }
}

function CreateRegKeyFromFile ($regKeyFileObject) {
  if($regKeyFileObject.execute -eq $true) {
    Write-Debug ("Creating registry entry from file " + $env:ChocolateyPackageFolder + "\" + $regKeyFileObject.path)
    regedit.exe /S ($env:ChocolateyPackageFolder + "\" + $regKeyFileObject.path)
  }
}

function CreateInstallerParameters ($instParamObject) {
  if($instParamObject.execute -eq $true) {
    Write-Debug "Checking Installer Components for: "; Write-Debug $instParamObject.value
    $global:installerComponents += $instParamObject.value
    $global:installerComponents += ","
    Write-Debug ("These are the current installerComponents: " + $installerComponents + ".")
  }
}

function CopyPackageRessources ($PackageRessourcePathObject) {
  if($PackageRessourcePathObject.execute -eq $true) {
    Write-Debug ("Preparing Package Ressource Path for: " + $PackageRessourcePathObject.value + ".")
    $tempPath = ($env:ChocolateyPackageFolder + "\" + ($PackageRessourcePathObject.value).ToString())
    if((Test-Path $tempPath) -eq $false) {
      Write-Debug ("The ressource object " + $PackageRessourcePathObject.value + " does not exist.")
      return
    }
    # We create the target directory first as Copy-Item behaves differently when the target path doesn't exist as directory
    New-Item $PackageRessourcePathObject.installPath -ItemType Directory -Force
    if($tempPath -Match "zip") {
      Get-ChocolateyUnzip -FileFullPath "$tempPath" -Destination $PackageRessourcePathObject.installPath
    } else {
      Move-Item -Path $tempPath -Destination $PackageRessourcePathObject.installPath -Force -Recurse
      Write-Debug ("Copying all files from Package\" + $PackageRessourcePathObject.value + " to " + $PackageRessourcePathObject.installPath + ".")
    }
  }
}

function CreateTxtFiles ($txtFileObject) {
  if($txtFileObject.execute -eq $true) {
    Write-Debug ("Creating txt file in " + $txtFileObject.key + " with the content: " + $txtFileObject.value)
    Remove-Item $txtFileObject.key -ErrorAction SilentlyContinue
    New-Item $txtFileObject.key -ItemType file -Force
    [System.IO.File]::WriteAllLines($txtFileObject.key, $txtFileObject.value, (New-Object ("System.Text." + $txtFileObject.encoding + "Encoding") $txtFileObject.bom))
  }
}

function RunInstallerWithPackageParametersObject ($packageParameterObject) {
  $installerPathViaPP = (Test-Path $pp["InstallerPath"])
  ## Does the following check work for 32 bit installers??
  $installerEmbedded = ((Test-Path ($env:ChocolateyPackageFolder + "\" + $packageParameterObject.file)) -And ($packageParameterObject.file64 -ne $null) -And (Test-Path ($env:ChocolateyPackageFolder + "\" + $packageParameterObject.file64)))
  $installerDownload = (($packageParameterObject.url -ne $null) -Or ($packageParameterObject.url64 -ne $null))
  $installerDownloadExe = (($packageParameterObject.url -ne $null) -And ($packageParameterObject.url).EndsWith(".exe"))
  Write-Debug ("This is the InstallerPath Variable: " + $pp["InstallerPath"])
  Write-Debug ("Installer embedded path: " + ($env:ChocolateyPackageFolder + "\" + $packageParameterObject.file))
  Write-Debug ("Installer embedded path: " + ($env:ChocolateyPackageFolder + "\" + $packageParameterObject.file64))
  if(Test-Path variable:packageParameterObject.url]) {Write-Debug ("Installer url: " + $packageParameterObject.url)}
  if($installerDownloadExe -eq $true) {Write-Debug "Installer is exe"}

  if($installerPathViaPP -eq $true) {
    $packageParameterObject["file"] = $fileLocation
    Install-ChocolateyInstallPackage @packageParameterObject # https://chocolatey.org/docs/helpers-install-chocolatey-install-package
  }

  if($installerEmbedded -eq $true) {
    $packageParameterObject.file = ($env:ChocolateyPackageFolder + "\" + $packageParameterObject.file)
    $packageParameterObject.file64 = ($env:ChocolateyPackageFolder + "\" + $packageParameterObject.file64)
    Write-Debug ("Installer embedded path: " + $packageParameterObject.file)
    Install-ChocolateyInstallPackage @packageParameterObject
  }

  if($installerDownload -eq $true -And $installerPathViaPP -eq $false) {
    if($installerDownloadExe -eq $true) { "Installer is exe, running now..."
      Install-ChocolateyPackage @packageParameterObject # https://chocolatey.org/docs/helpers-install-chocolatey-package
    } else { Write-Debug "Installer inside zip"; Write-Debug ("UnzipLocation is: " + $packageParameterObject.unzipLocation)
      $packageParameterObject["file"] = $fileLocation
      Install-ChocolateyZipPackage @packageParameterObject
      Install-ChocolateyInstallPackage @packageParameterObject
    }
  }
}

function RemoveInstallerObjects ($packageParameterObject) {
  if($packageParameterObject.file) {
    if(Test-Path ($packageParameterObject.file)) {
      Remove-Item $packageParameterObject.file -Force -ErrorAction SilentlyContinue
    }
  } elseif($packageParameterObject.file64) {
    if(Test-Path ($packageParameterObject.file64) -ErrorAction Ignore){
      Remove-Item $packageParameterObject.file64 -Force -ErrorAction SilentlyContinue
    }
  }
}

function RemoveTemporaryFiles ($PackageRessourcePathObject) {
  Write-Debug ("Deleting temp package files: " + $env:ChocolateyPackageFolder + "\" + $PackageRessourcePathObject.value + " and" + $env:ChocolateyPackageFolder + "\" + $PackageRessourcePathObject.value + ".txt")
  Remove-Item ($env:ChocolateyPackageFolder + "\" + $PackageRessourcePathObject.value) -Force -ErrorAction SilentlyContinue
  Remove-Item ($env:ChocolateyPackageFolder + "\" + $PackageRessourcePathObject.value + ".txt") -Force -ErrorAction SilentlyContinue
}

function HandlePackageArgs ($packageParameterObject) {
  if ($installerComponents) {
    $packageParameterObject["silentArgs"] += " /Components=$installerComponents" #Q2 2018 the '' around $installerComponents have to be removed because Zebra2.8 installer ignores parameters
  }
  Write-Debug "This are the silentArgs"; Write-Debug $packageParameterObject["silentArgs"]
}

# Creates txt file containing all zip files and where they have been copied to.
# Also checks whether extracted zip content should be uninstalled or not (.delete=$false).
function WriteUninstallData ($PackageRessourcePathObject) {
  if($PackageRessourcePathObject.delete -ne $false) {
    foreach($line in Get-Content ($env:ChocolateyPackageFolder + "\" + $PackageRessourcePathObject.value + ".txt")) {
      Add-Content -Path ($env:ChocolateyPackageFolder + "\uninstall.txt") -Value ($PackageRessourcePathObject.installPath + "\" + $line)
    }
  }
}

# Creates/Overwrites bit-aware variables, so x86 files can use the same path-var on both platforms
function CreateBitAwareVariables () {
  if($osBitness -eq 64) {
    $global:vst2x86BitAware = "$vst2x86_64Path"
    $global:vst3x86BitAware = "$vst3x86_64Path"
    $global:aaxx86BitAware  = "$aaxx86_64Path"
  }
  if($osBitness -eq 32) {
    $global:vst2x86BitAware = "$vst2Path"
    $global:vst3x86BitAware = "$vst3Path"
    $global:aaxx86BitAware  = "$aaxPath"
  }
}

# Uninstaller Functions
function Test-Registry ($path, $value) {
  try {
    Get-ItemProperty -Path $Path | Select-Object -ExpandProperty $Value -ErrorAction Stop | Out-Null
    return $true
  } catch {
    return $false
  }
}

function DeleteFolders ($folderPath) {
  if($folderPath -eq $null -or $folderPath -eq "") {
    Write-Debug "Der übergebene Pfad ist null."
    return
  }
  if(Test-Path -path "$folderPath") {
    Write-Debug "$folderPath will be deleted."
    # "Archive" seems to be a lnk file
    if((Get-Item ($folderPath).ToString()).Attributes.ToString() -Match "Archive" ) {
      Remove-Item $folderPath -Force -Recurse
    } else {
      [System.IO.Directory]::Delete("$folderPath", $true)
    }
  }
}

# Checks, if a folder is empty and if true, deletes that folder. Also checks three levels upwards. -Force is not necessary.
function DeleteFoldersIfEmpty ($folderPath) {
  Write-Debug ("Checking " + $folderPath)
  if($folderPath -eq $null) {
    Write-Debug "Path variable is null."
    return
  }
  if($folderPath -eq "") {
    Write-Debug "Path to be deleted is an empty string."
    return
  }
  if((Test-Path -Path "$folderPath") -eq $false) {
    Write-Debug "$folderPath does not exist."
    return
  }
  $parentFolder = (Get-ItemProperty $folderPath).Parent.FullName
  $parentParentFolder = (Get-ItemProperty $folderPath).Parent.Parent.FullName
  $folderList = "$folderPath", "$parentFolder", "$parentParentFolder"
  Foreach ($path in $folderList) {
    Write-Debug ("Checking if path exists and is empty: " + $path)
    if($path -ne '' -and $path -ne $null) {
      if(((Test-Path "$path") -and (Get-ChildItem "$path") -eq $null)) {
        Write-Debug "$path is empty and will be deleted."
        Remove-Item "$path"
      }
    }
  }
}

function DeleteDataFromTxtFile ($uninstallFile) {
  Foreach($path in Get-Content $uninstallFile) {
    Write-Debug ("About to delete " + $path)
    if(Test-Path $path -pathType leaf) {
      $directoryName = (Get-Item $path).DirectoryName
      Write-Debug "$path will be deleted."
      Remove-Item "$path" -Force
      DeleteFoldersIfEmpty($directoryName)
    }
    if(Test-Path $path -pathType container) {
      $parentDirectory = (Get-ItemProperty $path).Parent.FullName
      Write-Debug "$path will be deleted."
      #Remove-Item -Recurse -Force fails if folder contains a symlink -.-
      [System.IO.Directory]::Delete("$path", $true)
      DeleteFoldersIfEmpty($parentDirectory)
    }
  }
}

# Removes all registry entires in a given registry file from the registry
function DeleteRegKeysFromFile ($file) {
  if(Test-Path $file) {
    (Get-Content $file) -replace "^\[","[-" | out-file $file
    regedit.exe /S ($file)
  }
}

# Deletes a registry key or property, if it exists and is not marked as delete=$false
function DeleteRegKeyFromObjects ($regKey) {
  Write-Debug ("About to delete Registry Entry: " + $regKey.path + $regKey.key)
  if($regKey.ContainsKey("delete")) {
    return
  }
  if((Test-Path -Path $regKey.path) -eq $false) {
    Write-Debug ("Registry Entry does not exist anymore.")
    return
  }
  if(($regKey.key -eq "")) {
    Write-Debug ("Deleting " + $regKey.path)
    Remove-Item -Path $regKey.path -Force
  }
  if ($regKey.key -ne "") {
    Write-Debug ("Deleting Key " + $regKey.key + " at Path " + $regKey.path)
    Remove-ItemProperty -Path $regKey.path -Name $regKey.key -Force -ErrorAction SilentlyContinue
  }
}

# Deletes a file, if it exists
function DeleteFile ($pathToFile) {
  if(Test-Path $pathToFile) {
    Write-Debug ("Deleting " + $pathToFile)
    Remove-Item $pathToFile -Force
  }
}

# Packaging functions
function CreateFileList ($packagePaths, $targetPath) {
  Remove-Item ($targetPath + ".txt") -ErrorAction SilentlyContinue
  Foreach ($path in $packagePaths) {
    #Add-Content -Path ($targetPath + ".txt") -Value (Get-ChildItem -Path "$path" -Recurse -Name) -Force
    Add-Content -Path ($targetPath + ".txt") -Value ((Get-Item -Path "$path").Name) -Force
  }
}
function CreateDataArchive ($packagePaths, $targetPath) {
  Compress-Archive $packagePaths $targetPath -CompressionLevel Optimal -Force
}

function CreateDataArchiveAndFileList ($packagePaths, $targetPath) {
  if ((Test-Path $targetPath) -eq $false ) {
    New-Item $targetPath -Force
    Remove-Item $targetPath -Force
  }
  CreateDataArchive $packagePaths $targetPath
  CreateFileList $packagePaths $targetPath
}
